// Constants
const LOCAL_STORAGE_KEYS = {
    INSTAGRAM_COOKIES: 'easylive-instagram-cookies',
    INSTAGRAM_TOKEN: 'easylive-instagram-token',
    INSTAGRAM_REQUEST: 'easylive-instagram-request',
    INSTAGRAM_HEADERS: 'easylive-instagram-headers'
};

const MESSAGE_TYPES = {
    GET_VERSION: 'EASYLIVE_GET_VERSION',
    CHECK_INSTALLED: 'EASYLIVE_CHECK_INSTALLED',
    CLEAN_TOKEN: 'EASYLIVE_CLEAN_TOKEN',
    GET_INSTAGRAM_COOKIES: 'EASYLIVE_GET_INSTAGRAM_COOKIES',
    GET_TOKEN: 'EASYLIVE_GET_TOKEN',
    GET_HEADERS: 'EASYLIVE_GET_HEADERS',
    INSTAGRAM_RCV_TOKEN: 'EASYLIVE_INSTAGRAM_RCV_TOKEN',
    FACEBOOK_RCV_TOKEN: 'EASYLIVE_FACEBOOK_RCV_TOKEN',
    GET_INSTAGRAM_DATA: 'EASYLIVE_GET_INSTAGRAM_DATA',
};

// Utility functions
const StorageHelper = {
    get: (keys) => new Promise(resolve => {
        chrome.storage.local.get(keys, resolve);
    }),

    set: (items) => new Promise(resolve => {
        chrome.storage.local.set(items, resolve);
    }),

    remove: (keys) => new Promise(resolve => {
        chrome.storage.local.remove(keys, resolve);
    })
};

const CookieHelper = {
    getAllInstagramCookies: () => new Promise(resolve => {
        chrome.cookies.getAll({domain: "instagram.com"}, resolve);
    }),

    validateUserId: (cookies) => {
        return cookies?.ds_user_id?.trim() !== '';
    },

    cookiesToObject: (cookies) => {
        return cookies.reduce((acc, cookie) => {
            acc[cookie.name] = cookie.value;
            return acc;
        }, {});
    }
};

const HeaderHelper = {
    getBrowserHeaders: () => {
        const userAgent = navigator.userAgent;
        const chromeVersion = userAgent.match(/Chrome\/(\d+)/)?.[1] || "130";

        return {
            "secChUa": `"Chromium";v="${chromeVersion}", "Google Chrome";v="${chromeVersion}", "Not?A_Brand";v="99"`,
            "userAgent": userAgent,
        };
    }
};

// Message handlers
const ExternalMessageHandlers = {
    async [MESSAGE_TYPES.GET_VERSION]() {
        return chrome.runtime.getManifest().version;
    },

    async [MESSAGE_TYPES.CHECK_INSTALLED]() {
        return true;
    },

    async [MESSAGE_TYPES.CLEAN_TOKEN]() {
        await StorageHelper.remove([
            LOCAL_STORAGE_KEYS.INSTAGRAM_TOKEN,
            LOCAL_STORAGE_KEYS.INSTAGRAM_REQUEST,
            LOCAL_STORAGE_KEYS.INSTAGRAM_COOKIES
        ]);
        return true;
    },

    async [MESSAGE_TYPES.GET_INSTAGRAM_COOKIES]() {
        const result = await StorageHelper.get(LOCAL_STORAGE_KEYS.INSTAGRAM_COOKIES);
        const cookies = result[LOCAL_STORAGE_KEYS.INSTAGRAM_COOKIES];
        return cookies && CookieHelper.validateUserId(cookies) ? cookies : null;
    },

    async [MESSAGE_TYPES.GET_TOKEN]() {
        const result = await StorageHelper.get(LOCAL_STORAGE_KEYS.INSTAGRAM_TOKEN);
        return result[LOCAL_STORAGE_KEYS.INSTAGRAM_TOKEN];
    },

    async [MESSAGE_TYPES.GET_HEADERS]() {
        const headers = HeaderHelper.getBrowserHeaders();
        await StorageHelper.set({
            [LOCAL_STORAGE_KEYS.INSTAGRAM_HEADERS]: headers
        });
        return headers;
    },

    async [MESSAGE_TYPES.GET_INSTAGRAM_DATA]() {
        const [cookieResult, headerResult] = await Promise.all([
            StorageHelper.get(LOCAL_STORAGE_KEYS.INSTAGRAM_COOKIES),
            StorageHelper.get(LOCAL_STORAGE_KEYS.INSTAGRAM_HEADERS)
        ]);

        const cookies = cookieResult[LOCAL_STORAGE_KEYS.INSTAGRAM_COOKIES];
        const headers = headerResult[LOCAL_STORAGE_KEYS.INSTAGRAM_HEADERS] || HeaderHelper.getBrowserHeaders();

        if (!cookies || !CookieHelper.validateUserId(cookies)) {
            return null;
        }

        return {
            cookies,
            headers,
            timestamp: Date.now()
        };
    },
    async ['EASYLIVE_INIT_CONNECTION']() {
        const manifest = chrome.runtime.getManifest();
        return {
            name: manifest.name,
            manifest_version: manifest.manifest_version,
            description: manifest.description,
            id: chrome.runtime.id,
            version: manifest.version
        };
    }
};

const InternalMessageHandlers = {
    async [MESSAGE_TYPES.INSTAGRAM_RCV_TOKEN]() {
        await updateInstagramStorage();
    },

    async [MESSAGE_TYPES.FACEBOOK_RCV_TOKEN](request) {
        await StorageHelper.set({
            [LOCAL_STORAGE_KEYS.INSTAGRAM_TOKEN]: request.token,
            [LOCAL_STORAGE_KEYS.INSTAGRAM_REQUEST]: request
        });
    }
};

// Core functionality
async function updateInstagramStorage() {
    const cookies = await CookieHelper.getAllInstagramCookies();
    const cookieObj = CookieHelper.cookiesToObject(cookies);
    const headerObj = HeaderHelper.getBrowserHeaders();

    await StorageHelper.set({
        [LOCAL_STORAGE_KEYS.INSTAGRAM_COOKIES]: cookieObj,
        [LOCAL_STORAGE_KEYS.INSTAGRAM_HEADERS]: headerObj
    });
}

// Event listeners
chrome.runtime.onMessageExternal.addListener((request, sender, sendResponse) => {
    const handler = ExternalMessageHandlers[request.type];
    if (handler) {
        handler(request).then(sendResponse);
        return true; // Will send response asynchronously
    }
    sendResponse(undefined);
});

chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    const handler = InternalMessageHandlers[request.type];
    if (handler) {
        handler(request).then(sendResponse);
        return true; // Will send response asynchronously
    }
});

chrome.runtime.onInstalled.addListener((details) => {
    if (details.reason === chrome.runtime.OnInstalledReason.INSTALL ||
        details.reason === chrome.runtime.OnInstalledReason.UPDATE) {
        chrome.tabs.create({url: 'https://easylive.ir/blog/how-to-use-easylive-extension/'});
    }
});